import std.math;
import std.string;

alias uint Pixel;

S TDOT(S, T) (T A, T B) {
  return A.x * B.x + A.y * B.y + A.z * B.z;
}

void TNORMALIZE(T) (inout T A) {
  float l = 1 / sqrt(A.x * A.x + A.y * A.y + A.z * A.z);

  A.x *= l;
  A.y *= l;
  A.z *= l;
}

S TLENGTH(S, T) (T A) {
  return sqrt(A.x * A.x + A.y * A.y + A.z * A.z);
}

S TSQRLENGTH(S, T) (T A) {
  return A.x * A.x + A.y * A.y + A.z * A.z;
}

S TSQRDISTANCE(S, T) (T A, T B) {
  return (A.x - B.x) * (A.x - B.x) + (A.y - B.y) * (A.y - B.y) + (A.z - B.z) * (A.z - B.z);
}

alias float Scalar;
alias vector3 Vector;

Scalar DOT(Vector A, Vector B) {
  return A.x * B.x + A.y * B.y + A.z * B.z;
}

void NORMALIZE(inout Vector A) {
  float l = 1 / sqrt(A.x * A.x + A.y * A.y + A.z * A.z);

  A.x *= l;
  A.y *= l;
  A.z *= l;
}

Scalar LENGTH(Vector A) {
  return sqrt(A.x * A.x + A.y * A.y + A.z * A.z);
}

Scalar SQRLENGTH(Vector A) {
  return A.x * A.x + A.y * A.y + A.z * A.z;
}

Scalar SQRDISTANCE(Vector A, Vector B) {
  return (A.x - B.x) * (A.x - B.x) + (A.y - B.y) * (A.y - B.y) + (A.z - B.z) * (A.z - B.z);
}


const float EPSILON = 0.001;
const int TRACEDEPTH = 6;

const float PI = 3.141592653589793238462;

struct vector3 {
  static vector3 opCall( float a_X, float a_Y, float a_Z ) {
    vector3 r;
    r.x = a_X;
    r.y = a_Y;
    r.z = a_Z;
    return r;
  }

  static vector3 opCall( vector3 v ) {
    vector3 r;
    r.x = v.x;
    r.y = v.y;
    r.z = v.z; 
    return r;
  }

  void Set( float a_X, float a_Y, float a_Z ) {
    x = a_X; 
    y = a_Y;
    z = a_Z;
  }
  void Set( vector3 v ) {
    x = v.x;
    y = v.y;
    z = v.z;
  }
  void Normalize() {
    float l = 1.0f / Length();
    x *= l;
    y *= l;
    z *= l;
  }

  float Length() {
    return cast(float)sqrt( x * x + y * y + z * z );
  }
  float SqrLength() {
    return x * x + y * y + z * z;
  }
  float Dot( vector3 a_V ) {
    return x * a_V.x + y * a_V.y + z * a_V.z;
  }
  vector3 Cross( vector3 b ) {
    return vector3( y * b.z - z * b.y, z * b.x - x * b.z, x * b.y - y * b.x );
  }
  void opAddAssign( vector3 a_V ) {
    x += a_V.x;
    y += a_V.y;
    z += a_V.z;
  }
  void opSubAssign( vector3 a_V ) {
    x -= a_V.x;
    y -= a_V.y;
    z -= a_V.z;
  }
  void opMulAssign( float f ) {
    x *= f;
    y *= f;
    z *= f;
  }
  void opMulAssign( vector3 a_V ) {
    x *= a_V.x;
    y *= a_V.y;
    z *= a_V.z;
  }
  vector3 opNegate() {
    return vector3( -x, -y, -z );
  }
  vector3 opAdd( vector3 v2 ) {
    return vector3( x + v2.x, y + v2.y, z + v2.z );
  }
  vector3 opSub( vector3 v2 ) {
    return vector3( x - v2.x, y - v2.y, z - v2.z );
  }
  vector3 opMul( float f ) {
    return vector3( x * f, y * f, z * f );
  }
  vector3 opMul_r( float f ) {
    return vector3( x * f, y * f, z * f );
  }
  vector3 opMul( vector3 v2 ) {
    return vector3( x * v2.x, y * v2.y, z * v2.z );
  }
  char[] toString() {
    return format(z, ",", y, ",", z);
  }

  float x = 0;
  float y = 0;
  float z = 0;
  alias x r;
  alias y g;
  alias z b;

}

class plane {
  this() {
  }
  this( vector3 a_Normal, float a_D ) {
    N = a_Normal;
    D = a_D;
  }
  vector3 N;
  float D = 0;
}

alias vector3 Color;
