// Don't look here. The interesting stuff is elsewhere.

import common;
import raytracer;
import scene;
import surface;
import std.string;
import std.stdio;
import windows;

const int SCRWIDTH = 800;
const int SCRHEIGHT = 600;

static WNDCLASS wc;
static HWND wnd;
static char bitmapbuffer[BITMAPINFO.sizeof + 16];
static BITMAPINFO * bh;
HDC window_hdc;
Surface surface_ = null;
Pixel * buffer = null;
Engine tracer = null;
bool running = true;

extern (Windows) static LRESULT WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {

  int result = 0, keycode = 0;

  switch (message) {
  case WM_PAINT:
    if (!buffer) {
      break;
    }
    StretchDIBits( window_hdc, 0, 0, SCRWIDTH, SCRHEIGHT, 0, 0, SCRWIDTH, SCRHEIGHT, buffer, bh, DIB_RGB_COLORS, SRCCOPY );
    ValidateRect( wnd, null );
    break;
  case WM_KEYDOWN:
    if ((wParam & 0xFF) != 27) {
      break;
    }
  case WM_CLOSE:
    ReleaseDC( wnd, window_hdc );
    DestroyWindow( wnd );
    SystemParametersInfo( SPI_SETSCREENSAVEACTIVE, 1, null, 0 );
    running = false;
    break;
  default:
    result = DefWindowProc(hWnd, message, wParam, lParam);
  }
  return result;
}

extern (C) void gc_init();
extern (C) void gc_term();
version (DigitalMars) {
  extern (C) void _minit();
}
extern (C) void _moduleCtor();
extern (C) void _moduleUnitTests();

version (GNU) {
  int main() {
    return WinMain(null, null, null, 0);
  }
}

extern (Windows) int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
  version (DigitalMars) {
    gc_init();                    // initialize garbage collector
    _minit();                     // initialize module constructor table
    _moduleCtor();                // call module constructors
  }

  RECT rect;
  wc.style = CS_OWNDC | CS_VREDRAW | CS_HREDRAW;
  wc.lpfnWndProc = &WndProc;
  wc.cbClsExtra = wc.cbWndExtra = 0;
  wc.hInstance = null;
  wc.hIcon = null;
  wc.hCursor = LoadCursor(null, IDC_ARROW);
  wc.hbrBackground = null;
  wc.lpszMenuName = null;
  wc.lpszClassName = "raytracer";
  ATOM registered = RegisterClass(&wc);
  assert(registered);
  rect.left = rect.top = 0;
  rect.right = SCRWIDTH;
  rect.bottom = SCRHEIGHT;
  AdjustWindowRect( &rect, WS_POPUP | WS_SYSMENU | WS_CAPTION, 0 );
  rect.right -= rect.left;
  rect.bottom -= rect.top;
  wnd = CreateWindowEx( 0, toStringz("raytracer"), toStringz("raytracer"),
          WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX & ~WS_THICKFRAME,
          CW_USEDEFAULT, CW_USEDEFAULT, rect.right, rect.bottom, null, null, null, null );
  assert(wnd);
  ShowWindow(wnd, SW_NORMAL);
  for ( int cc = 0; cc < BITMAPINFOHEADER.sizeof + 16; cc++ ) {
    bitmapbuffer[cc] = 0;
  }
  bh = cast(BITMAPINFO *)bitmapbuffer.ptr;
  bh.bmiHeader.biSize = BITMAPINFOHEADER.sizeof;
  bh.bmiHeader.biPlanes = 1;
  bh.bmiHeader.biBitCount = 32;
  bh.bmiHeader.biCompression = BI_BITFIELDS;
  bh.bmiHeader.biWidth = SCRWIDTH;
  bh.bmiHeader.biHeight = -SCRHEIGHT;
  (cast(uint *)bh.bmiColors)[0] = 255 << 16;
  (cast(uint *)bh.bmiColors)[1] = 255 << 8;
  (cast(uint *)bh.bmiColors)[2] = 255;
  window_hdc = GetDC(wnd);
  assert(window_hdc);
  SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 0, null, 0);
  // prepare output canvas
  surface_ = new Surface( SCRWIDTH, SCRHEIGHT );
  buffer = surface_.GetBuffer();
  surface_.Clear( 0 );
  surface_.InitCharset();
  surface_.Print( "timings:", 2, 2, 0xffffffff );
  // prepare renderer
  tracer = new Engine();
  tracer.GetScene().InitScene();
  tracer.SetTarget( surface_.GetBuffer(), SCRWIDTH, SCRHEIGHT );
  int tposX = 60;
  int tposY = 2;
  // go
  int fstart = GetTickCount();
  tracer.InitRender();
  while (!tracer.Render() && running) {
    DrawWindow();
  }
  int ftime = GetTickCount() - fstart;
  char t[] = "00:00.000";
  version (GNU) {
    t = t.dup;
  }
  t[6] = (ftime / 100) % 10 + '0';
  t[7] = (ftime / 10) % 10 + '0';
  t[8] = (ftime % 10) + '0';
  int secs = (ftime / 1000) % 60;
  int mins = (ftime / 60000) % 100;
  t[3] = ((secs / 10) % 10) + '0';
  t[4] = (secs % 10) + '0';
  t[1] = (mins % 10) + '0';
  t[0] = ((mins / 10) % 10) + '0';
  surface_.Print( t, tposX, tposY, 0xffffffff );
  
  while (running) {
     DrawWindow();
  }

  version (DigitalMars) {
    gc_term();
  }

  return 1;
}

void DrawWindow() {
  MSG message;
  HACCEL haccel = null;

  InvalidateRect( wnd, null, TRUE );
  SendMessage( wnd, WM_PAINT, 0, 0 );
  while (PeekMessage( &message, wnd, 0, 0, PM_REMOVE )) {
    if (TranslateAccelerator( wnd, haccel, &message ) == 0) {
      TranslateMessage( &message );
      DispatchMessage( &message );
    }
  }
  Sleep( 1 );
}
